@@ -4,7 +4,7 @@ use std::ops::AddAssign;
 use arrow_buffer::BooleanBuffer;
 use num_traits::AsPrimitive;
 use vortex_array::arrays::PrimitiveArray;
-use vortex_array::compute::{FilterFn, filter};
+use vortex_array::compute::{FilterKernel, filter};
 use vortex_array::validity::Validity;
 use vortex_array::variants::PrimitiveArrayTrait;
 use vortex_array::{Array, ArrayRef, Canonical, ToCanonical};
@@ -18,7 +18,7 @@ use crate::{RunEndArray, RunEndEncoding};
 
 const FILTER_TAKE_THRESHOLD: f64 = 0.1;
 
-impl FilterFn<&RunEndArray> for RunEndEncoding {
+impl FilterKernel for RunEndEncoding {
     fn filter(&self, array: &RunEndArray, mask: &Mask) -> VortexResult<ArrayRef> {
         match mask {
             Mask::AllTrue(_) => Ok(array.to_array()),
@@ -140,7 +140,7 @@ mod tests {
                 .ends()
                 .to_primitive()
                 .unwrap()
-                .as_slice::<u64>(),
+                .as_slice::<u8>(),
             [2, 4]
         );
         assert_eq!(
@@ -168,7 +168,7 @@ mod tests {
                 .ends()
                 .to_primitive()
                 .unwrap()
-                .as_slice::<u64>(),
+                .as_slice::<u8>(),
             [1, 2, 3]
         );
         assert_eq!(
