@@ -3,6 +3,7 @@ use std::sync::Arc;
 
 use vortex_dtype::{DType, ExtDType};
 use vortex_error::{VortexResult, vortex_bail};
+use vortex_mask::Mask;
 use vortex_scalar::ExtScalar;
 
 use crate::arrays::ExtensionArray;
@@ -82,6 +83,14 @@ impl ArrayBuilder for ExtensionBuilder {
         array.storage().append_to_builder(self.storage.as_mut())
     }
 
+    fn ensure_capacity(&mut self, capacity: usize) {
+        self.storage.ensure_capacity(capacity)
+    }
+
+    fn set_validity(&mut self, validity: Mask) {
+        self.storage.set_validity(validity);
+    }
+
     fn finish(&mut self) -> ArrayRef {
         let storage = self.storage.finish();
         ExtensionArray::new(self.ext_dtype(), storage).into_array()
