@@ -1,10 +1,10 @@
 use std::sync::Arc;
 
-use vortex_error::{vortex_err, VortexResult};
+use vortex_error::{VortexResult, vortex_bail, vortex_err};
 use vortex_flatbuffers::FlatBuffer;
 
 use crate::field::Field;
-use crate::{flatbuffers as fb, DType, StructDType};
+use crate::{DType, StructDType, flatbuffers as fb};
 
 /// Convert name references in projection list into index references.
 ///
@@ -20,7 +20,7 @@ pub fn resolve_field<'a, 'b: 'a>(fb: fb::Struct_<'b>, field: &'a Field) -> Vorte
                 .position(|name| name == &**n)
                 .ok_or_else(|| vortex_err!("Unknown field name {n}"))
         }
-        Field::Index(i) => Ok(*i),
+        _ => vortex_bail!("Only field names are supported for now"),
     }
 }
 
