@@ -1,14 +1,16 @@
-use std::sync::{Arc, Mutex};
+use std::sync::Arc;
 
 use futures::{StreamExt, pin_mut};
-use moka::future::CacheBuilder;
 use vortex_error::{VortexExpect, VortexResult};
 use vortex_io::{Dispatch, InstrumentedReadAt, IoDispatcher, VortexReadAt};
 use vortex_layout::segments::{SegmentEvents, SegmentSource};
 use vortex_metrics::VortexMetrics;
 
 use crate::driver::CoalescedDriver;
-use crate::segments::{CachedSegmentSource, InMemorySegmentCache, SegmentCache};
+use crate::segments::{
+    InitialReadSegmentCache, MokaSegmentCache, SegmentCache, SegmentCacheMetrics,
+    SegmentCacheSourceAdapter,
+};
 use crate::{FileType, SegmentSourceFactory, SegmentSpec, VortexFile, VortexOpenOptions};
 
 /// A type of Vortex file that supports any [`VortexReadAt`] implementation.
@@ -27,10 +29,9 @@ impl VortexOpenOptions<GenericVortexFile> {
     /// Open a file using the provided [`VortexReadAt`] implementation.
     pub fn file() -> Self {
         Self::new(Default::default())
-            .with_segment_cache(Arc::new(InMemorySegmentCache::new(
-                // For now, use a fixed 1GB overhead.
-                CacheBuilder::new(1 << 30),
-            )))
+            // Start with an initial in-memory cache of 256MB.
+            // TODO(ngates): would it be better to default to a home directory disk cache?
+            .with_segment_cache(Arc::new(MokaSegmentCache::new(256 << 20)))
             .with_initial_read_size(Self::INITIAL_READ_SIZE)
     }
 
@@ -39,13 +40,22 @@ impl VortexOpenOptions<GenericVortexFile> {
         self
     }
 
-    pub async fn open<R: VortexReadAt + Send>(self, read: R) -> VortexResult<VortexFile> {
+    pub async fn open<R: VortexReadAt + Send + Sync>(self, read: R) -> VortexResult<VortexFile> {
+        let read = Arc::new(read);
         let footer = self.read_footer(&read).await?;
 
+        let segment_cache = Arc::new(SegmentCacheMetrics::new(
+            InitialReadSegmentCache {
+                initial: self.initial_read_segments,
+                fallback: self.segment_cache,
+            },
+            self.metrics.clone(),
+        ));
+
         let segment_source_factory = Arc::new(GenericVortexFileIo {
-            read: Mutex::new(read),
+            read,
             segment_map: footer.segment_map().clone(),
-            segment_cache: self.segment_cache,
+            segment_cache,
             options: self.options,
         });
 
@@ -58,27 +68,24 @@ impl VortexOpenOptions<GenericVortexFile> {
 }
 
 struct GenericVortexFileIo<R> {
-    read: Mutex<R>,
+    read: Arc<R>,
     segment_map: Arc<[SegmentSpec]>,
     segment_cache: Arc<dyn SegmentCache>,
     options: GenericFileOptions,
 }
 
-impl<R: VortexReadAt + Send> SegmentSourceFactory for GenericVortexFileIo<R> {
+impl<R: VortexReadAt + Send + Sync> SegmentSourceFactory for GenericVortexFileIo<R> {
     fn segment_source(&self, metrics: VortexMetrics) -> Arc<dyn SegmentSource> {
         // We use segment events for driving I/O.
         let (segment_source, events) = SegmentEvents::create();
 
         // Wrap the source to resolve segments from the initial read cache.
-        let segment_source = Arc::new(CachedSegmentSource::new(
+        let segment_source = Arc::new(SegmentCacheSourceAdapter::new(
             self.segment_cache.clone(),
             segment_source,
         ));
 
-        let read = InstrumentedReadAt::new(
-            self.read.lock().vortex_expect("poisoned lock").clone(),
-            &metrics,
-        );
+        let read = InstrumentedReadAt::new(self.read.clone(), &metrics);
 
         let driver = CoalescedDriver::new(
             read.performance_hint(),
@@ -95,7 +102,7 @@ impl<R: VortexReadAt + Send> SegmentSourceFactory for GenericVortexFileIo<R> {
                 async move {
                     // Drive the segment event stream.
                     let stream = driver
-                        .map(|coalesced_req| coalesced_req.launch(read.clone()))
+                        .map(|coalesced_req| coalesced_req.launch(&read))
                         .buffer_unordered(io_concurrency);
                     pin_mut!(stream);
 
@@ -152,7 +159,7 @@ impl Default for GenericFileOptions {
     fn default() -> Self {
         Self {
             io_concurrency: 8,
-            io_dispatcher: IoDispatcher::default(),
+            io_dispatcher: IoDispatcher::shared(),
         }
     }
 }
