@@ -1,6 +1,4 @@
 mod cache;
-mod cached;
 pub(crate) mod writer;
 
 pub use cache::*;
-pub use cached::*;
