@@ -201,12 +201,13 @@ mod test {
             .await
             .unwrap();
 
-        let reader = AsyncIPCReader::try_new(Cursor::new(ipc_buffer), Default::default())
-            .await
-            .unwrap();
+        let reader =
+            AsyncIPCReader::try_new(Cursor::new(ipc_buffer), ArrayRegistry::canonical_only())
+                .await
+                .unwrap();
 
         assert_eq!(reader.dtype(), array.dtype());
-        let result = reader.into_array().await.unwrap().to_primitive().unwrap();
+        let result = reader.read_all().await.unwrap().to_primitive().unwrap();
         assert_eq!(array.as_slice::<i32>(), result.as_slice::<i32>());
     }
 }
