@@ -7,10 +7,18 @@ use crate::segments::SegmentWriter;
 /// A strategy for writing chunks of an array into a layout.
 // [layout writer]
 pub trait LayoutWriter: Send {
-    fn push_chunk(&mut self, segments: &mut dyn SegmentWriter, chunk: ArrayRef)
-    -> VortexResult<()>;
+    /// Push a chunk into the layout writer.
+    fn push_chunk(
+        &mut self,
+        segment_writer: &mut dyn SegmentWriter,
+        chunk: ArrayRef,
+    ) -> VortexResult<()>;
 
-    fn finish(&mut self, segments: &mut dyn SegmentWriter) -> VortexResult<Layout>;
+    /// Flush any buffered chunks.
+    fn flush(&mut self, segment_writer: &mut dyn SegmentWriter) -> VortexResult<()>;
+
+    /// Write any final data (e.g. stats) and return the finished [`Layout`].
+    fn finish(&mut self, segment_writer: &mut dyn SegmentWriter) -> VortexResult<Layout>;
 }
 // [layout writer]
 
@@ -26,24 +34,26 @@ pub trait LayoutWriterExt: LayoutWriter {
     /// Push a single chunk into the layout writer and return the finished [`Layout`].
     fn push_one(
         &mut self,
-        segments: &mut dyn SegmentWriter,
+        segment_writer: &mut dyn SegmentWriter,
         chunk: ArrayRef,
     ) -> VortexResult<Layout> {
-        self.push_chunk(segments, chunk)?;
-        self.finish(segments)
+        self.push_chunk(segment_writer, chunk)?;
+        self.flush(segment_writer)?;
+        self.finish(segment_writer)
     }
 
     /// Push all chunks of the iterator into the layout writer and return the finished
     /// [`Layout`].
     fn push_all<I: IntoIterator<Item = VortexResult<ArrayRef>>>(
         &mut self,
-        segments: &mut dyn SegmentWriter,
+        segment_writer: &mut dyn SegmentWriter,
         iter: I,
     ) -> VortexResult<Layout> {
         for chunk in iter.into_iter() {
-            self.push_chunk(segments, chunk?)?
+            self.push_chunk(segment_writer, chunk?)?
         }
-        self.finish(segments)
+        self.flush(segment_writer)?;
+        self.finish(segment_writer)
     }
 }
 
